/*
 * ======================================================================
 * FunctionEvaluation.java: This program evaluates and prints values
 * for a function y(x), x ranging from -4 to 10 in increments of 0.2.
 *
 * Three trouble spots need handling:
 *
 * 1. At x = 0, the expression x/sin(x) evaluates to not-a-number.
 * 2. At x = 2, there is a divide by zero.
 * 3. The increment 0.2 cannot be stored exactly inside the computer.
 *    Hence, tests for relational equality should be based on tolerances.
 *    
 * Strategy:
 *
 * This program evaluates y(x), tests for error conditions in the result,
 * and then prints an appropriate message.
 *
 * Note: Program demonstrates use of special error constants, Double.NaN,
 *       Double.NEGATIVE_INFINITY and Double.POSITIVE_INFINITY.
 *
 * By: Mark Austin                                          November 2007
 * ======================================================================
 */

package basics;

import java.lang.Math;
import java.util.*;
import java.io.*;
import java.text.*;

public class DemoFunctionEvaluation {
    public static void main( String args[] ) {
 	double dY, dX;

	// Print header and setup table...

	System.out.println("");
	System.out.println("     x | Function y(x)");
	System.out.println("----------------------");

	// Loop over range of x values....

        dX = -4.0;
        while ( dX <= 10.0 ) {

           // Evaluate y(x) ....

           dY = (Math.pow (dX,4.0) + (dX/Math.sin(dX)))/(dX-2.0);

           // Detect and handle error conditions apriori.
           // Otherwise evaluate and print x, y(x).

           if ( dY == Double.POSITIVE_INFINITY ) 
             System.out.printf("%6.2f      Positive Infinity\n", dX );
           else if ( dY == Double.NEGATIVE_INFINITY ) 
             System.out.printf("%6.2f      Negative Infinity\n", dX );
           else if ( dY == Double.NaN ) 
             System.out.printf("%6.2f      Not a Number\n", dX );
           else 
             System.out.printf("%6.2f      %10.2f\n", dX, dY );

           // Increment value of dX.

           dX = dX + 0.25;
	}
    }
}
