/*
 *  =======================================================================
 *  TestLineSegment2D.java: Exercise methods in line segment class.
 * 
 *  Written By: Mark Austin                                      April 2019 
 *  =======================================================================
 */

package geometry;

import java.lang.Math;

public class TestLineSegment2D {
   public static void main( String args[] ) {
   double dX, dY;

      System.out.printf("LineSegment2D test program ...\n");
      System.out.printf("================================================ \n");

      // Create line segment s1 ...
      // -------------------------------

      LineSegment2D s1 = new LineSegment2D( 1.0, 2.0, 6.0, 7.0 );
      s1.getNode1().setName("n1");
      s1.getNode2().setName("n2");
      s1.setName("s1");

      // Create line segment s2 ...
      // -------------------------------

      Node2D n1 = new Node2D( 3.0, 0.0 );
      n1.setName( "n3" );
      Node2D n2 = new Node2D( 3.0, 2.0 );
      n2.setName( "n4" );

      LineSegment2D s2 = new LineSegment2D( n1, n2 );
      s2.setName("s2");

      // Create line segment s3 ...
      // -------------------------------

      LineSegment2D s3 = new LineSegment2D( 5.0, 1.0, 6.0, 7.0 );
      s3.getNode1().setName("n1");
      s3.getNode2().setName("n2");
      s3.setName("s3");

      // Print details of line segments.
      // -------------------------------

      System.out.println ( s1.toString() );

      System.out.println ( s2.toString() );

      System.out.println ( s3.toString() );

      // Creat test points ...
      // -----------------------------------------------------

      Node2D pt1 = new Node2D ( "pt1", 0.0,  0.0 );
      Node2D pt2 = new Node2D ( "pt2", 1.0,  0.0 );
      Node2D pt3 = new Node2D ( "pt3", 5.0,  1.0 );
      Node2D pt4 = new Node2D ( "pt4", 5.0,  6.0 );
      Node2D pt5 = new Node2D ( "pt5", 3.0,  7.0 );

      // Compute distance of a point from a length segment ...
      // -----------------------------------------------------

      System.out.printf("\n");
      System.out.printf("Compute distance of a point to line s1 ... \n");
      System.out.printf("-------------------------------------- ... \n");
      System.out.printf("\n");

      System.out.println( pt1.toString() + ": distance to s1 = " +
                          s1.distanceToPoint( pt1 ) );

      System.out.println( pt2.toString() + ": distance to s1 = " +
                          s1.distanceToPoint( pt2 ) );

      System.out.println( pt3.toString() + ": distance to s1 = " +
                          s1.distanceToPoint( pt3 ) );

      System.out.println( pt4.toString() + ": distance to s1 = " +
                          s1.distanceToPoint( pt4 ) );

      // Test to see if a node is on a line segment ...
      // -----------------------------------------------------

      System.out.printf("\n");
      System.out.printf("Test if point lies on line segment ... \n");
      System.out.printf("-------------------------------------- ... \n");
      System.out.printf("\n");

      System.out.println( pt1.toString() + ": is on s1 = " + s1.isPointOnLine( pt1 ) );
      System.out.println( pt2.toString() + ": is on s1 = " + s1.isPointOnLine( pt2 ) );
      System.out.println( pt3.toString() + ": is on s1 = " + s1.isPointOnLine( pt3 ) );
      System.out.println( pt4.toString() + ": is on s1 = " + s1.isPointOnLine( pt4 ) );
      System.out.println( pt5.toString() + ": is on s1 = " + s1.isPointOnLine( pt5 ) );

      System.out.printf("\n");
      System.out.printf("Test if point lies to right of line segment ... \n");
      System.out.printf("------------------------------------------- ... \n");
      System.out.printf("\n");

      System.out.println( pt1.toString() + ": is on s1 = " + s1.isPointRightOfLine( pt1 ) );
      System.out.println( pt2.toString() + ": is on s1 = " + s1.isPointRightOfLine( pt2 ) );
      System.out.println( pt3.toString() + ": is on s1 = " + s1.isPointRightOfLine( pt3 ) );
      System.out.println( pt4.toString() + ": is on s1 = " + s1.isPointRightOfLine( pt4 ) );
      System.out.println( pt5.toString() + ": is on s1 = " + s1.isPointRightOfLine( pt5 ) );

      System.out.printf("\n");
      System.out.printf("Test if first line segment touches or crosses second ... \n");
      System.out.printf("---------------------------------------------------- ... \n");
      System.out.printf("\n");

      System.out.println( "--- Does s1 touch (or cross) s2 --> " + s1.lineSegmentTouchesOrCrossesLine( s2 ));
      System.out.println( "--- Does s2 touch (or cross) s1 --> " + s2.lineSegmentTouchesOrCrossesLine( s1 ));

      System.out.printf("\n");
      System.out.println( "--- Does s3 touch (or cross) s1 --> " + s3.lineSegmentTouchesOrCrossesLine( s1 ));
      System.out.println( "--- Does s3 touch (or cross) s2 --> " + s3.lineSegmentTouchesOrCrossesLine( s2 ));

      System.out.printf("\n");
      System.out.printf("Test for intersection of bounding boxes ... \n");
      System.out.printf("---------------------------------------------------- ... \n");
      System.out.printf("\n");

      // Does bounding box for s1 overlap bounding box for s2 ...

      BoundingBox b1 = s1.getBoundingBox();
      BoundingBox b2 = s2.getBoundingBox();
      BoundingBox b3 = s3.getBoundingBox();

      System.out.println( "--- Does s1.boundingBox() intersect s2.boundingBox() --> " + b1.intersect( b2 ) );
      System.out.println( "--- Does s1.boundingBox() intersect s3.boundingBox() --> " + b1.intersect( b3 ) );
      System.out.println( "--- Does s2.boundingBox() intersect s3.boundingBox() --> " + b2.intersect( b3 ) );

      System.out.printf("\n");
      System.out.printf("Test for intersection of line segments ... \n");
      System.out.printf("---------------------------------------------------- ... \n");
      System.out.printf("\n");

      System.out.println( "--- Does s1 intersect s2 --> " + s1.intersect( s2 ) );
      System.out.println( "--- Does s1 intersect s3 --> " + s1.intersect( s3 ) );
      System.out.println( "--- Does s2 intersect s3 --> " + s2.intersect( s3 ) );

      System.out.printf("================================================ \n");
      System.out.printf("Done! ... \n");
   }
}
