
package lambda;

import java.util.*;
import java.util.function.Predicate;

public class TestPredicate02 {

   public static void main(String... args) {

      List<Box> inventory = Arrays.asList( new Box(80, "green"), new Box( 155, "green"), new Box(120, "red") );

      // Apply filter: TestPredicate02::isHeavyApple ...

      List<Box> greenApples = filter(inventory, TestPredicate02::isGreenApple );
      System.out.println(greenApples);

      // Apply filter: TestPredicate02::isHeavyApple ...

      List<Box> heavyApples = filter(inventory, TestPredicate02::isHeavyApple );
      System.out.println(heavyApples);

      // Apply filter: (Box a) -> "green".equals(a.getColor())) ...

      List<Box> greenApples2 = filter(inventory, (Box a) -> "green".equals(a.getColor()));
      System.out.println(greenApples2);

      // Apply filter: (Box a) -> a.getWeight() > 150) ...

      List<Box> heavyApples2 = filter(inventory, (Box a) -> a.getWeight() > 150);
      System.out.println(heavyApples2);

      // Apply filter: (Box a) -> a.getWeight() < 80 || "brown".equals(a.getColor())) ...

      List<Box> weirdApples = filter(inventory, (Box a) -> a.getWeight() < 80 || "brown".equals(a.getColor()));
      System.out.println(weirdApples);
  }

  // Evaluate constraints .....

  public static boolean isGreenApple(Box apple) {
    return "green".equals(apple.getColor());
  }

  public static boolean isHeavyApple(Box apple) {
    return apple.getWeight() > 150;
  }

   // Filter list of apples against predicate p ....

   public static List<Box> filter(List<Box> inventory, Predicate<Box> p) {
      List<Box> result = new ArrayList<>();
      for (Box apple : inventory) {
          if (p.test(apple) == true ) {
             result.add(apple);
          }
      }
      return result;
   }
}

class Box {
  private int weight = 0;
  private String color = "";

  public Box(int weight, String color) {
    this.weight = weight;
    this.color = color;
  }

  public Integer getWeight() {
    return weight;
  }

  public void setWeight(Integer weight) {
    this.weight = weight;
  }

  public String getColor() {
    return color;
  }

  public void setColor(String color) {
    this.color = color;
  }

  public String toString() {
    return "Apple{" + "color='" + color + '\'' + ", weight=" + weight + '}';
  }
}
