/* 
 *  ========================================================================
 *  PlotSigmoidFunction01.java: Simple linechart plot of sigmoid function.
 *  
 *  Written by: Mark Austin                                  September, 2020
 *  ========================================================================
 */ 

package neural;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.block.BlockBorder;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.chart.title.TextTitle;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

import java.lang.Math.*;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.EventQueue;
import java.awt.Font;

public class PlotSigmoidFunction01 extends JFrame {

    public PlotSigmoidFunction01() {
        createGUI();
    }

    private void createGUI() {

        XYDataset dataset = createDataset();
        JFreeChart chart  = createChart(dataset);

        ChartPanel chartPanel = new ChartPanel(chart);
        chartPanel.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));
        chartPanel.setBackground(Color.white);
        add(chartPanel);

        pack();
        setTitle("Plot of sigmoid function");
        setLocationRelativeTo(null);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

    // method to compute sigmoid function ...

    public double sigmoid ( double x ) {
      return 1/(1 + Math.exp(-x));
    }

    // Create x-y data set ...

    private XYDataset createDataset() {

        XYSeries series = new XYSeries("Sigmoid");

        // main loop to evaluate and print sigmoid function ...

        for ( double x = -10.0; x <= 10.0; x = x + 0.5 ) {
            series.add( x, sigmoid(x));
        }

        // add data series to collection of line plots ...

        XYSeriesCollection dataset = new XYSeriesCollection();
        dataset.addSeries(series);

        return dataset;
    }

    // Create chart ...

    private JFreeChart createChart(XYDataset dataset) {

        JFreeChart chart = ChartFactory.createXYLineChart(
             "sigmoid(x) vs x", "x", "sigmoid(x)", dataset,
             PlotOrientation.VERTICAL, true, true, false);

        XYPlot plot = chart.getXYPlot();

        XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
        renderer.setSeriesPaint(0, Color.RED);
        renderer.setSeriesStroke(0, new BasicStroke(2.0f));

        plot.setRenderer(renderer);
        plot.setBackgroundPaint(Color.white);

        plot.setRangeGridlinesVisible(true);
        plot.setRangeGridlinePaint(Color.BLACK);

        plot.setDomainGridlinesVisible(true);
        plot.setDomainGridlinePaint(Color.BLACK);

        chart.getLegend().setFrame(BlockBorder.NONE);

        chart.setTitle(new TextTitle("sigmoid(x) versus x",
                                     new Font("Serif", java.awt.Font.BOLD, 18) )
        );

        return chart;
    }

    public static void main(String[] args) {
        EventQueue.invokeLater(() -> {
            PlotSigmoidFunction01 ex = new PlotSigmoidFunction01();
            ex.setVisible(true);
        });
    }
}
