/**
  * ==========================================================================
  * DemoDataFormatOutput02.java: Java Program to demonstrate how to use
  * LocalDate, LocalTime, and LocalDateTime in Java 8.
  * 
  * Modified by: Mark Austin                                    November, 2021 
  * ==========================================================================
  */

package time;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.Month;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.time.format.FormatStyle;
import java.util.Locale;

public class DemoDateFormatOutput02 {
   public static void main(String[] args) {

      System.out.printf("Enter DemoDateFormatOutput02.main() ... \n" );
      System.out.printf("============================================================== \n" );

      // Formatting dates in Java 8

      System.out.printf("--- Create LocalDateTime object ... \n" );

      LocalDateTime now = LocalDateTime.now();

      System.out.printf("---   LocalDateTime.now() --> %s ... \n", now.toString() );

      // Example 1 - print date as 2016-06-16 (yyyy-MM-dd)

      String isoDate = now.format(DateTimeFormatter.ISO_DATE);
      System.out.printf("--- ISO date format: %s ...\n", isoDate);

      // Example 2 - print date as 20160616 (yyyyMMdd)

      String basicIsoDate = now.format(DateTimeFormatter.BASIC_ISO_DATE);
      System.out.printf("--- Basic ISO date format: %s ...\n", basicIsoDate);

      // Example 3 - formatting date in British or Indian date format dd-MM-yyyy

      String indianDateFormat = now.format(DateTimeFormatter.ofPattern("dd-MM-yyyy"));
      System.out.printf("--- Formatting in Indian date format: %s ...\n", indianDateFormat);

      // Example 4 - formatting date in USA date format (06-16-2016)

      DateTimeFormatter americanDateFormat = DateTimeFormatter.ofPattern("MM-dd-yyyy");
      String americanDate = now.format(americanDateFormat);
      System.out.printf("--- USA date format: %s ...\n", americanDate);

      // Example 5 - formatting in Japanese date format (16-06-16)

      DateTimeFormatter japan = DateTimeFormatter.ofPattern("yy-MM-dd");
      String japanese = now.format(japan);
      System.out.printf("--- Date in Japan dateformat: %s ...\n", japanese);

      // Example 6 - format in ISO date time format

      String isoDateTime = now.format(DateTimeFormatter.ISO_DATE_TIME);
      System.out.printf("--- Date in ISO date time format: %s ...\n", isoDateTime);

      // Example 7 - french date formatting 16. juin 2016

      DateTimeFormatter french = DateTimeFormatter.ofPattern("d. MMMM yyyy", new Locale("fr"));
      String frenchDate = now.format(french);
      System.out.printf("--- Date in French format: %s ...\n", frenchDate);

      // Example 8 - using short german date/time formatting (06.16.16 12:07)

      DateTimeFormatter formatter = DateTimeFormatter.ofLocalizedDateTime( FormatStyle.SHORT).withLocale(new Locale("de"));
      String germanDateTime = now.format(formatter);

      System.out.printf("--- Date in short German format: %s ...\n", germanDateTime);

      // Example 9 - Formatting time in ISO_TIME format

      String isoTime = now.format(DateTimeFormatter.ISO_TIME);
      System.out.printf("--- Time in ISO TIME format: %s ...\n", isoTime);

      // Example 10 - Printing date in long format e.g. 16th June 2016

      String longFormat = now.format(DateTimeFormatter.ofPattern("dd-MMM-yyyy"));
      System.out.printf("--- Date in long format: %s ...\n", longFormat);

      // Example 11 - Formatting date with timezone

      ZonedDateTime dateWithTimeZone = ZonedDateTime.of(now, ZoneId.of("Europe/London"));
      String withTimeZone = dateWithTimeZone .format(DateTimeFormatter.ISO_ZONED_DATE_TIME);

      System.out.printf("--- Date time with timezone: %s ...\n", withTimeZone);

      // Parsing text to Date in Java 8

      System.out.printf("--- Parse text to date (LocalDate) in Java 8 ...\n");

      LocalDate fromIsoDate       = LocalDate.parse("2015-02-23");
      LocalDate fromCustomPattern = LocalDate.parse("20-03-2017", DateTimeFormatter.ofPattern("dd-MM-yyyy"));

      System.out.printf("---   fromIsoDate: %s ...\n", fromIsoDate.toString() );
      System.out.printf("---   fromCustomPattern: %s ...\n", fromCustomPattern.toString() );

      // LocalDate invalidDate = LocalDate.parse("16-02-2018");
      // parsing text to LocalTime in Java 8

      System.out.printf("--- Parse text to local time (LocalTime) in Java 8 ...\n");

      LocalTime fromIsoTime = LocalTime.parse("12:07:43.048");
      LocalTime fromPattern = LocalTime.parse("11:06:32",
                              DateTimeFormatter.ofPattern("HH:mm:ss"));

      System.out.printf("---   fromIsoTime: %s ...\n", fromIsoTime.toString() );
      System.out.printf("---   fromPattern: %s ...\n", fromPattern.toString() );

      // Converting String LocalDateTime and ZonedDateTime in Java 8

      System.out.printf("--- Convert String to LocalDateTime and ZonedDateTime in Java 8 ...\n");

      LocalDateTime fromIsoDateTime      = LocalDateTime.parse("2016-06-16T13:12:38.954");
      ZonedDateTime fromIsoZonedDateTime = ZonedDateTime.parse("2016-06-16T13:12:38.954+01:00[Europe/London]");

      System.out.printf("============================================================== \n" );
      System.out.printf("Finished !! \n" );
   }
} 
